@@ -33,13 +33,14 @@ try:
     import gdata.calendar.service
     import gdata.contacts.service
     import gdata.docs.service
+    import gdata.youtube.service
 
     MODULES = {
         "GoogleCalendarTwoWay" : { "type": "dataprovider" },
         "PicasaTwoWay" :         { "type": "dataprovider" },
-        "YouTubeSource" :        { "type": "dataprovider" },    
+        "YouTubeTwoWay" :        { "type": "dataprovider" },    
         "ContactsTwoWay" :       { "type": "dataprovider" },
-#        "DocumentsSink" :        { "type": "dataprovider" },
+        "DocumentsSink" :        { "type": "dataprovider" },
     }
     log.info("Module Information: %s" % Utils.get_module_information(gdata, None))
 except (ImportError, AttributeError):
@@ -50,6 +51,7 @@ except (ImportError, AttributeError):
 FORMAT_STRING = "%Y-%m-%dT%H:%M:%S"
 
 class _GoogleBase:
+    _configurable_ = True
     def __init__(self, service):
         self.username = ""
         self.password = ""
@@ -587,6 +589,26 @@ class PicasaTwoWay(_GoogleBase, Image.ImageTwoWay):
         except Exception, e:
             raise Exceptions.SyncronizeError("Picasa Upload Error.")
 
+    def _replace_photo(self, id, uploadInfo):
+        try:
+            gphoto = self.gphoto_dict[id]
+
+            gphoto.title = atom.Title(text=uploadInfo.name)
+            gphoto.summary = atom.Summary(text=uploadInfo.caption)
+            gphoto.media = gdata.media.Group()
+            gphoto.media.keywords = gdata.media.Keywords()
+            if uploadInfo.tags:
+                gphoto.media.keywords.text = ",".join("%s" % (str(t)) for t in uploadInfo.tags)
+        
+            gphoto = self.service.UpdatePhotoMetadata(gphoto)
+        
+            # This should be done just only the photo itself has changed
+            gphoto = self.service.UpdatePhotoBlob(gphoto, uploadInfo.url)
+
+            return Rid(uid=gphoto.gphoto_id.text)
+        except Exception, e:
+            raise Exceptions.SyncronizeError("Picasa Update Error.")
+
     def _get_album(self):
         for name,album in self._get_albums():
             if name == self.albumName:
@@ -772,6 +794,7 @@ class ContactsTwoWay(_GoogleBase,  DataProvider.TwoWay):
     _description_ = _("Sync your Gmail contacts")
     _category_ = conduit.dataproviders.CATEGORY_OFFICE
     _module_type_ = "twoway"
+    _in_type_ = "contact"
     _out_type_ = "contact"
     _icon_ = "contact-new"
 
@@ -990,7 +1013,7 @@ class ContactsTwoWay(_GoogleBase,  DataProvider.TwoWay):
         
 class _GoogleDocument:
     def __init__(self, doc):
-        self.id = doc.id.text
+        self.id = doc.GetSelfLink().href
         #raw text version link
         self.raw = doc.content.src
         #edit link
@@ -1000,11 +1023,14 @@ class _GoogleDocument:
         self.authorEmail = doc.author[0].email.text
         self.type = doc.category[0].label
         
+        self.editLink = doc.GetEditLink().href
+        
         self.updated = convert_madness_to_datetime(doc.updated.text)
         self.docid = self.get_document_id(self.link)
         
     # Parses the document id out of the alternate link url, the atom feed
-    # doesn't actually provide the document id
+    # doesn't actually provide the document id. Need it for downloading in
+    # different formats
     @staticmethod
     def get_document_id(LUID):
         parsed_url = urlparse.urlparse(LUID)
@@ -1014,7 +1040,7 @@ class _GoogleDocument:
         
     def __str__(self):
         return "%s:%s by %s (modified:%s) (id:%s)" % (self.type,self.title,self.authorName,self.updated,self.docid)
-
+    
 class DocumentsSink(_GoogleBase,  DataProvider.DataSink):
     """
     Contacts GData provider
@@ -1024,7 +1050,7 @@ class DocumentsSink(_GoogleBase,  DataProvider.DataSink):
     _name_ = _("Google Documents")
     _description_ = _("Sync your Google Documents")
     _category_ = conduit.dataproviders.CATEGORY_OFFICE
-    _module_type_ = "twoway"
+    _module_type_ = "sink"
     _out_type_ = "contact"
     _icon_ = "applications-office"
     
@@ -1065,14 +1091,47 @@ class DocumentsSink(_GoogleBase,  DataProvider.DataSink):
             log.info("Unknown document format")
             return None
 
-        return entry.id.text
+        return entry.GetSelfLink().href
+        
+    def _replace_document(self, LUID, f):
+        # Doesnt support updating easily, trash the old one, and make a new one
+        # http://code.google.com/p/gdata-issues/issues/detail?id=277
+        #self.delete(LUID)
+        #upload the file
+        #pf = self._get_proxyfile(
+        #                self._upload_document(f))
+        #if pf:
+        #    return pf.get_rid()
+        #raise Exceptions.SynchronizeError("Error Uploading")
+        
+        # The follwing says one can replace like this
+        # http://code.google.com/p/goofs/source/browse/trunk/goofs/src/goofs/backend.py#197
+        
+        name,ext = f.get_filename_and_extension()
+        ext = ext[1:].upper()
+
+        ms = gdata.MediaSource(
+                    file_path=f.get_local_uri(),
+                    content_type=gdata.docs.service.SUPPORTED_FILETYPES[ext])
+        
+        doc = self.service.Get(LUID)        
+        doc = self.service.Put(
+                doc,
+                doc.GetEditLink().href,
+                media_source=ms,
+                extra_headers = {'Slug':name})
+                
+        pf = self._get_proxyfile(doc.GetSelfLink().href)
+        if pf:
+            return pf.get_rid()
+        raise Exceptions.SynchronizeError("Error Replacing")
         
     def _get_all_documents(self):
         docs = {}
         feed = self.service.GetDocumentListFeed()
         if feed.entry:
             for xmldoc in feed.entry:
-                docs[xmldoc.id.text] = _GoogleDocument(xmldoc)
+                docs[xmldoc.GetSelfLink().href] = _GoogleDocument(xmldoc)
         return docs
         
     def _get_document(self, LUID):
@@ -1091,6 +1150,19 @@ class DocumentsSink(_GoogleBase,  DataProvider.DataSink):
 
         return _GoogleDocument(xmldoc)
         
+    def _get_proxyfile(self, LUID):
+        if LUID:
+            gdoc = self._get_document(LUID)
+            if gdoc:
+                f = File.ProxyFile(
+                            URI=gdoc.raw,
+                            name=gdoc.title,
+                            modified=gdoc.updated,
+                            size=None)
+                f.set_UID(LUID)
+                return f
+        return None
+        
     def _download_doc(self, googleDoc):
         docid = googleDoc.docid
     
@@ -1148,43 +1220,46 @@ class DocumentsSink(_GoogleBase,  DataProvider.DataSink):
     def get(self, LUID):
         pass
 
-#    def put(self, doc, overwrite, LUID=None):            
-#        #Check if we have already uploaded the document
-#        if LUID != None:
-#            info = self._get_photo_info(LUID)
-#            #check if a photo exists at that UID
-#            if info != None:
-#                if overwrite == True:
-#                    #replace the photo
-#                    return self._replace_photo(LUID, uploadInfo)
-#                else:
-#                    #Only upload the photo if it is newer than the Remote one
-#                    url = self._get_raw_photo_url(info)
-#                    remoteFile = File.File(url)
-#
-#                    #this is a limited test for equality type comparison
-#                    comp = photo.compare(remoteFile,True)
-#                    log.debug("Compared %s with %s to check if they are the same (size). Result = %s" % 
-#                            (photo.get_filename(),remoteFile.get_filename(),comp))
-#                    if comp != conduit.datatypes.COMPARISON_EQUAL:
-#                        raise Exceptions.SynchronizeConflictError(comp, photo, remoteFile)
-#                    else:
-#                        return conduit.datatypes.Rid(uid=LUID)
-#
-#        log.debug("Uploading Photo URI = %s, Mimetype = %s, Original Name = %s" % (photoURI, mimeType, originalName))
-#
-#        #upload the file
-#        return self._upload_photo (uploadInfo)
+    def put(self, f, overwrite, LUID=None):
+        DataProvider.DataSink.put(self, f, overwrite, LUID)       
+        #Check if we have already uploaded the document
+        if LUID != None:
+            gdoc = self._get_document(LUID)
+            #check if a doc exists at that UID
+            if gdoc != None:
+                if overwrite == True:
+                    #replace the document
+                    return self._replace_document(LUID, f)
+                else:
+                    #Only upload the doc if it is newer than the Remote one
+                    remoteFile = self._get_proxyfile(LUID)
+
+                    #compare based on mtime
+                    comp = f.compare(remoteFile)
+                    log.debug("Compared %s with %s to check if they are the same (mtime). Result = %s" % 
+                            (f.get_filename(),remoteFile.get_filename(),comp))
+                    if comp != conduit.datatypes.COMPARISON_EQUAL:
+                        raise Exceptions.SynchronizeConflictError(comp, photo, remoteFile)
+                    else:
+                        return conduit.datatypes.Rid(uid=LUID)
+
+        log.debug("Uploading Document")
+
+        #upload the file
+        pf = self._get_proxyfile(
+                        self._upload_document(f))
+        if pf:
+            return pf.get_rid()
+            
+        raise Exceptions.SynchronizeError("Error Uploading")
 
     def delete(self, LUID):
         DataProvider.DataSink.delete(self, LUID)
-        self._login()
-        #get the gdata contact from google
-        #try:
-        #    gc = self.service.Get(LUID, converter=gdata.contacts.ContactEntryFromString)
-        #    self.service.DeleteContact(gc.GetEditLink().href)
-        #except gdata.service.RequestError, e:
-        #    log.warn("Error deleting: %s" % e)        
+        gdoc = self._get_document(LUID)
+        if gdoc:
+            self.service.Delete(gdoc.editLink)
+            return True
+        return False
 
     def configure(self, window):
         """
@@ -1194,6 +1269,10 @@ class DocumentsSink(_GoogleBase,  DataProvider.DataSink):
 
         def make_combo(widget, docType, val, values):
             cb = widget.get_widget("%sCombo" % docType)
+            
+            #FIXME: Make these unsensitive when download works better
+            cb.set_property("sensitive", False)
+            
             store = gtk.ListStore(str)
             cell = gtk.CellRendererText()
             
@@ -1231,7 +1310,30 @@ class DocumentsSink(_GoogleBase,  DataProvider.DataSink):
             self._set_password(password.get_text())
         dlg.destroy()   
 
-class YouTubeSource(DataProvider.DataSource):
+class VideoUploadInfo:
+    """
+    Upload information container, this way we can add info
+    and keep the _upload_info method on the VideoSink retain
+    its API
+    
+    Default category for videos is People/Blogs (for lack of a better
+    general category).
+    
+    Default name and description are placeholders, or the generated XML is invalid
+    as the corresponding elements automatically self-close.
+    
+    Default keyword is "miscellaneous" as the upload fails if no keywords
+    are specified.
+    """
+    def __init__ (self, url, mimeType, name=None, keywords=None, description=None, category=None):
+        self.url = url
+        self.mimeType = mimeType
+        self.name = name or _("Unknown")
+        self.keywords = keywords or (_("miscellaneous"),)
+        self.description = description or _("No description.")
+        self.category = category or "People" # Note: don't translate this; it's an identifier
+
+class YouTubeTwoWay(_GoogleBase, DataProvider.TwoWay):
     """
     Downloads YouTube videos using the gdata API.
     Based on youtube client from : Philippe Normand (phil at base-art dot net)
@@ -1240,7 +1342,8 @@ class YouTubeSource(DataProvider.DataSource):
     _name_ = _("YouTube")
     _description_ = _("Sync data from YouTube")
     _category_ = conduit.dataproviders.CATEGORY_MISC
-    _module_type_ = "source"
+    _module_type_ = "twoway"
+    _in_type_ = "file/video"
     _out_type_ = "file/video"
     _icon_ = "youtube"
 
@@ -1254,49 +1357,44 @@ class YouTubeSource(DataProvider.DataSource):
     UPLOAD_URL="http://uploads.gdata.youtube.com/feeds/api/users/%(username)s/uploads"
 
     def __init__(self, *args):
-        DataProvider.DataSource.__init__(self)
-        self.entries = None
-        self.username = ""
-        self.max_downloads = 0
-        #filter type {0 = mostviewed, 1 = toprated, 2 = user}
-        self.filter_type = 0
-        #filter user type {0 = upload, 1 = favorites}
-        self.user_filter_type = 0
+        youtube_service = gdata.youtube.service.YouTubeService()
+        youtube_service.client_id = self.UPLOAD_CLIENT_ID
+        youtube_service.developer_key = self.UPLOAD_DEVELOPER_KEY
 
-    def initialize(self):
-        return True
+        _GoogleBase.__init__(self,youtube_service)
+        DataProvider.TwoWay.__init__(self)
+
+        self.entries = None
+        self.max_downloads = 0
+        #filter type {0 = mostviewed, 1 = toprated, 2 = user upload, 3 = user favorites}
+        self.filter_type = 0
 
     def configure(self, window):
         tree = Utils.dataprovider_glade_get_widget (
                 __file__,
                 "youtube-config.glade",
-                "YouTubeSourceConfigDialog") 
+                "YouTubeTwoWayConfigDialog") 
 
-        dlg = tree.get_widget ("YouTubeSourceConfigDialog")
+        dlg = tree.get_widget ("YouTubeTwoWayConfigDialog")
         mostviewedRb = tree.get_widget("mostviewed")
         topratedRb = tree.get_widget("toprated")
-        byuserRb = tree.get_widget("byuser")
-        user_frame = tree.get_widget("frame")
         uploadedbyRb = tree.get_widget("uploadedby")
         favoritesofRb = tree.get_widget("favoritesof")
-        user = tree.get_widget("user")
-        maxdownloads = tree.get_widget("maxdownloads")
-
-        byuserRb.connect("toggled", self._filter_user_toggled_cb, user_frame)
+        max_downloads = tree.get_widget("maxdownloads")
+        username = tree.get_widget("username")
+        password = tree.get_widget("password")
 
         if self.filter_type == 0:
             mostviewedRb.set_active(True)
         elif self.filter_type == 1:
             topratedRb.set_active(True)
+        elif self.filter_type == 2:
+            uploadedbyRb.set_active(True)
         else:
-            byuserRb.set_active(True)
-            user_frame.set_sensitive(True)
-            if self.user_filter_type == 0:
-                uploadedbyRb.set_active(True)
-            else:
-                favoritesofRb.set_active(True)
-            user.set_text(self.username)
-        maxdownloads.set_value(self.max_downloads)
+            favoritesofRb.set_active(True)
+        max_downloads.set_value(self.max_downloads)
+        username.set_text(self.username)
+        password.set_text(self.password)
 
         response = Utils.run_dialog(dlg, window)
         if response == True:
@@ -1304,35 +1402,75 @@ class YouTubeSource(DataProvider.DataSource):
                 self.filter_type = 0
             elif topratedRb.get_active():
                 self.filter_type = 1
-            else:
+            elif uploadedbyRb.get_active():
                 self.filter_type = 2
-                if uploadedbyRb.get_active():
-                    self.user_filter_type = 0
-                else:
-                    self.user_filter_type = 1
-                self.username = user.get_text()
-            self.max_downloads = int(maxdownloads.get_value())
+            else:
+                self.filter_type = 3
+            self.max_downloads = int(max_downloads.get_value())
+            self.username = username.get_text()
+            self.password = password.get_text()
 
         dlg.destroy()
 
+    def _get_video_info (self, id):
+        if self.entries.has_key(id):
+            return self.entries[id]
+        else:
+            return None
+
+    def _extract_video_id (self, uri):
+        return uri.split ("/").pop ()
+
+    def _do_login(self):
+        # The YouTube login URL is slightly different to the normal Google one
+        self.service.ClientLogin(self.username, self.password, auth_service_url="https://www.google.com/youtube/accounts/ClientLogin")
+
+    def _upload_video (self, uploadInfo):
+        try:
+            self.gvideo = gdata.youtube.YouTubeVideoEntry()
+            self.gvideo.media = gdata.media.Group(
+                                title = gdata.media.Title(text=uploadInfo.name),
+                                description = gdata.media.Description(text=uploadInfo.description),
+                                category = gdata.media.Category(text=uploadInfo.category),
+                                keywords = gdata.media.Keywords(text=','.join(uploadInfo.keywords)))
+
+            gvideo = self.service.InsertVideoEntry(
+                                self.gvideo,
+                                uploadInfo.url)
+            return Rid(uid=self._extract_video_id(gvideo.id.text))
+        except Exception, e:
+            raise Exceptions.SyncronizeError("YouTube Upload Error.")
+
+    def _replace_video (self, LUID, uploadInfo):
+        try:
+            self.gvideo = self.service.GetYouTubeVideoEntry(video_id=LUID)
+            self.gvideo.media = gdata.media.Group(
+                                title = gdata.media.Title(text=uploadInfo.name),
+                                description = gdata.media.Description(text=uploadInfo.description),
+                                category = gdata.media.Category(text=uploadInfo.category),
+                                keywords = gdata.media.Keywords(text=','.join(uploadInfo.keywords)))
+
+            gvideo = self.service.UpdateVideoEntry(self.gvideo)
+            return Rid(uid=self._extract_video_id(gvideo.id.text))
+        except Exception, e:
+            raise Exceptions.SyncronizeError("YouTube Update Error.")
+
     def refresh(self):
-        DataProvider.DataSource.refresh(self)
+        DataProvider.TwoWay.refresh(self)
 
         self.entries = {}
         try:
-            feedUrl = ""
             if self.filter_type == 0:
                 videos = self._most_viewed ()
             elif self.filter_type == 1:
                 videos = self._top_rated ()
+            elif self.filter_type == 2:
+                videos = self._videos_upload_by (self.username)
             else:
-                if self.user_filter_type == 0:
-                    videos = self._videos_upload_by (self.username)
-                else:
-                    videos = self._favorite_videos (self.username)
+                videos = self._favorite_videos (self.username)
 
             for video in videos:
-                self.entries[video.title.text] = self._get_flv_video_url (video.link[0].href)
+                self.entries[self._extract_video_id(video.id.text)] = video.link[0].href
         except Exception, err:
             log.debug("Error getting/parsing feed (%s)" % err)
             raise Exceptions.RefreshError
@@ -1341,8 +1479,8 @@ class YouTubeSource(DataProvider.DataSource):
         return self.entries.keys()
 
     def get(self, LUID):
-        DataProvider.DataSource.get(self, LUID)
-        url = self.entries[LUID]
+        DataProvider.TwoWay.get(self, LUID)
+        url = self._get_flv_video_url(self.entries[LUID])
         log.debug("Title: '%s', Url: '%s'"%(LUID, url))
 
         f = Video.Video(URI=url)
@@ -1352,24 +1490,59 @@ class YouTubeSource(DataProvider.DataSource):
 
         return f
 
+    def put(self, video, overwrite, LUID=None):
+        """
+        Based off the ImageTwoWay put method.
+        Accepts a VFS file. Must be made local.
+        I also store a MD5 of the video's URI to check for duplicates
+        """
+        DataProvider.TwoWay.put(self, video, overwrite, LUID)
+
+        self._login()
+
+        originalName = video.get_filename()
+        #Gets the local URI (/foo/bar). If this is a remote file then
+        #it is first transferred to the local filesystem
+        videoURI = video.get_local_uri()
+        mimeType = video.get_mimetype()
+        keywords = video.get_tags ()
+        description = video.get_description()
+
+        uploadInfo = VideoUploadInfo(videoURI, mimeType, originalName, keywords, description)
+
+        #Check if we have already uploaded the video
+        if LUID != None:
+            url = self._get_video_info(LUID)
+            #Check if a video exists at that UID
+            if url != None:
+                if overwrite == True:
+                    #Replace the video
+                    return self._replace_video(LUID, uploadInfo)
+                else:
+                    #We can't do an equality test by size, since YouTube reencodes videos
+                    #on upload, so we'll just do nothing.
+                    return conduit.datatypes.Rid(uid=LUID)
+
+        log.debug("Uploading video URI = %s, Mimetype = %s, Original Name = %s" % (videoURI, mimeType, originalName))
+
+        #Upload the file
+        return self._upload_video (uploadInfo)
+
     def finish(self, aborted, error, conflict):
-        DataProvider.DataSource.finish(self)
-        self.files = None
+        DataProvider.TwoWay.finish(self)
+        self.entries = None
 
     def get_configuration(self):
         return {
             "filter_type"       :   self.filter_type,
-            "user_filter_type"  :   self.user_filter_type,
+            "max_downloads"     :   self.max_downloads,
             "username"          :   self.username,
-            "max_downloads"     :   self.max_downloads
+            "password"          :   self.password
         }
 
     def get_UID(self):
         return Utils.get_user_string()
 
-    def _filter_user_toggled_cb (self, toggle, frame):
-        frame.set_sensitive(toggle.get_active())
-
     def _format_url (self, url):
         if self.max_downloads > 0:
             url = ("%s?max-results=%d" % (url, self.max_downloads))
@@ -1380,19 +1553,19 @@ class YouTubeSource(DataProvider.DataSource):
         return service.Get(feed % params)
 
     def _top_rated(self):
-        url = self._format_url ("%s/top_rated" % YouTubeSource.STD_FEEDS)
+        url = self._format_url ("%s/top_rated" % YouTubeTwoWay.STD_FEEDS)
         return self._request(url).entry
 
     def _most_viewed(self):
-        url = self._format_url ("%s/most_viewed" % YouTubeSource.STD_FEEDS)
+        url = self._format_url ("%s/most_viewed" % YouTubeTwoWay.STD_FEEDS)
         return self._request(url).entry
 
     def _videos_upload_by(self, username):
-        url = self._format_url ("%s/%s/uploads" % (YouTubeSource.USERS_FEED, username))
+        url = self._format_url ("%s/%s/uploads" % (YouTubeTwoWay.USERS_FEED, username))
         return self._request(url).entry
 
     def _favorite_videos(self, username):
-        url = self._format_url ("%s/%s/favorites" % (YouTubeSource.USERS_FEED, username))
+        url = self._format_url ("%s/%s/favorites" % (YouTubeTwoWay.USERS_FEED, username))
         return self._request(url).entry
 
     # Generic extract step
@@ -1403,7 +1576,7 @@ class YouTubeSource(DataProvider.DataSource):
         data = doc.read()
 
         # extract video name
-        match = YouTubeSource.VIDEO_NAME_RE.search(data)
+        match = YouTubeTwoWay.VIDEO_NAME_RE.search(data)
         if match is not None:
             video_name = match.group(1)
 
